<?php
require dirname(__FILE__) . "/bifitKassaTools.php";

class BifitKassa
{
	const MODULE_ID = "bifit.kassa";

	public function __construct()
	{
	}

	static function __saveOrder($order)
	{
		Bitrix\Main\Localization\Loc::loadMessages(__FILE__);

		$mid = self::MODULE_ID;

		// log
		$log = "## " . GetMessage("BIFIT_LOG_GETTING_A_TOKEN") . "\n";

		// Get token
		$data = BifitKassaTools::__getToken($order->login, $order->password);
		$token = $data->access_token;

		// log
		if ($token) {
			$log .=
				"## " .
				GetMessage("BIFIT_LOG_GOT_A_TOKEN") .
				": " .
				$token .
				"\n";
		} else {
			$log .= "## " . GetMessage("BIFIT_LOG_TOKEN_ERROR") . "\n";
			$log .=
				"## " .
				GetMessage("BIFIT_ERROR") .
				": " .
				$data->error .
				"\n" .
				"## " .
				GetMessage("BIFIT_ERROR_DESC") .
				": " .
				$data->error_description;

			BifitKassaTools::__saveLogfile($log, $order->number);

			return false;
		}

		// log
		$log .=
			"## " . GetMessage("BIFIT_LOG_CHECK_AVAILABILITY_PRODUCT") . "\n";

		// parameters for ordering
		$onlineOrderItems = "";

		// form order data
		foreach ($order->products as $key_id => $product) {
			// address for POST request
			$url =
				"https://kassa.bifit.com/cashdesk-api/v1/protected/nomenclatures/external_id/" .
				$product->id .
				"?organization_id=" .
				$order->org_id;

			// link data
			$data = false;

			// headers
			$headers = [
				"Content-Type: application/x-www-form-urlencoded",
				"Authorization: Bearer " . $token
			];

			$prod = json_decode(
				BifitKassaTools::__loadData($url, $headers, $data, "GET")
			);

			if ($prod == null) {
				// CREATE A NEW PRODUCT

				// log
				$log .=
					"## " .
					GetMessage("BIFIT_LOG_PRODUCT_VS_ID") .
					$product->id .
					" " .
					GetMessage("BIFIT_LOG_NOT_FOUND_IN_BIFIT_DATABASE") .
					"\n";
				$log .= "## " . GetMessage("BIFIT_LOG_ADD_NEW_PRODUCT") . "\n";

				// address for POST request
				$url =
					"https://kassa.bifit.com/cashdesk-api/v1/protected/nomenclatures";

				// link data
				$data = "{";
				$data .= '"item":{';
				$data .=
					'"vatValue":' .
					($order->vat_mode == 1
						? $order->vat_value
						: $product->tax_value) .
					",";
				$data .= '"vendorCode":"' . $product->code . '",';
				$data .= '"name":"' . $product->name . '",';
				$data .= '"externalId":"' . $product->id . '",';
				$data .= '"sellingPrice":' . $product->price;
				$data .= "},";
				$data .= '"organizationId":"' . $order->org_id . '"';
				$data .= "}";

				// headers
				$headers = [
					"Content-Type: application/json",
					"Authorization: Bearer " . $token
				];

				// log
				$log .= GetMessage("BIFIT_LOG_DATA_FOR_REQUEST") . ":" . "\n";
				$log .= $data . "\n";

				// get data from request
				$tov = json_decode(
					BifitKassaTools::__loadData($url, $headers, $data)
				);

				// log
				if (!is_object($tov) && !is_array($tov)) {
					$log .=
						"## " .
						GetMessage("BIFIT_LOG_SUCC_ADD_NEW_PRODUCT") .
						$tov .
						"\n";
				} else {
					$log .=
						"## " .
						GetMessage("BIFIT_LOG_ERROR_WHEN_ADDED_NEW_PRODUCT") .
						":";
					$log .= BifitKassaTools::__checkResponce($tov) . "\n";
				}
			} else {
				$tov = $prod->id;

				// log
				$log .=
					"## " .
					GetMessage("BIFIT_LOG_PRODUCT_VS_ID") .
					$product->id .
					" " .
					GetMessage("BIFIT_LOG_FOUND_IN_BIFIT_DATABASE_VS_ID") .
					$tov .
					"\n";
			}

			if (isset($tov) && !is_object($tov) && !is_array($tov)) {
				// add nomenclatureId to product
				$order->products[$key_id]->nomenclature_id = $tov;

				// display product data in the order
				$onlineOrderItems .= $onlineOrderItems != '' ? ', ' : '';
				$onlineOrderItems .= '{';
				$onlineOrderItems .= '"nomenclatureId":' . $tov . ',';
				$onlineOrderItems .= '"paymentSubject":"PRODUCT",';
				$onlineOrderItems .= '"name":"' . $product->name . '",';
				$onlineOrderItems .= '"price":' . (float) $product->price . ',';
				$onlineOrderItems .=
					'"vatValue":' .
					($order->vat_mode == 1
						? $order->vat_value
						: $product->tax_value) .
					',';
				$onlineOrderItems .= '"quantity":' . $product->count . ',';
				$onlineOrderItems .= '"vendorCode":"' . $product->code . '"';
				$onlineOrderItems .= '}';
			}
		}

		// delivery details
		if ($order->delivery_cost > 0) {
			// address for POST request
			$url =
				"https://kassa.bifit.com/cashdesk-api/v1/protected/nomenclatures/external_id/" .
				"delivery_" .
				$order->delivery_id .
				"?organization_id=" .
				$order->org_id;

			// link data
			$data = false;

			// headers
			$headers = [
				"Content-Type: application/x-www-form-urlencoded",
				"Authorization: Bearer " . $token
			];

			$delivery = json_decode(
				BifitKassaTools::__loadData($url, $headers, $data, "GET")
			);

			// log
			$log .=
				"## " . GetMessage("BIFIT_LOG_CHECK_DELIVERY_SERVICE") . "\n";

			if ($delivery == null) {
				// CREATE A NEW SERVICE

				// log
				$log .=
					"## " .
					GetMessage("BIFIT_LOG_DELIVERY_SERVICE_NOT_FOUND") .
					"\n";
				$log .=
					"## " .
					GetMessage("BIFIT_LOG_ADD_NEW_DELIVERY_SERVICE") .
					"\n";

				// address for POST request
				$url =
					"https://kassa.bifit.com/cashdesk-api/v1/protected/nomenclatures";

				// link data
				$data = "{";
				$data .= '"item":{';
				$data .=
					'"vatValue":' .
					($order->vat_mode == 1
						? $order->vat_value
						: $order->delivery_tax_value) .
					",";
				$data .= '"name":"' . GetMessage("BIFIT_DELIVERY") . '",';
				$data .= '"externalId":"delivery_' . $order->delivery_id . '",';
				$data .= '"sellingPrice":' . $order->delivery_cost;
				$data .= "},";
				$data .= '"organizationId":"' . $order->org_id . '"';
				$data .= "}";

				// headers
				$headers = [
					"Content-Type: application/json",
					"Authorization: Bearer " . $token
				];

				// log
				$log .= GetMessage("BIFIT_LOG_DATA_FOR_REQUEST") . ":" . "\n";
				$log .= $data . "\n";

				// get data from request
				$tov = json_decode(
					BifitKassaTools::__loadData($url, $headers, $data)
				);

				// log
				if (!is_object($tov) && !is_array($tov)) {
					$log .=
						"## " .
						GetMessage("BIFIT_LOG_SUCC_ADD_NEW_DELIVERY_SERVICE") .
						$tov .
						"\n";
				} else {
					$log .=
						"## " .
						GetMessage(
							"BIFIT_LOG_ERROR_WHEN_ADDED_NEW_DELIVERY_SERVICE"
						) .
						"\n";
					$log .= BifitKassaTools::__checkResponce($tov) . "\n";
				}
			} else {
				$tov = $delivery->id;

				// log
				$log .=
					"## " .
					GetMessage(
						"BIFIT_LOG_DELIVERY_SERVICE_ALREADY_EXIST_VS_ID"
					) .
					$tov .
					"\n";
			}

			if (isset($tov) && !is_object($tov) && !is_array($tov)) {
				// add nomenclatureId to delivery
				$order->delivery_nomenclature_id = $tov;

				// display data of delivery method in order
				$onlineOrderItems .= $onlineOrderItems != "" ? "," : "";
				$onlineOrderItems .= "{";
				$onlineOrderItems .=
					'"nomenclatureId":' .
					$order->delivery_nomenclature_id .
					",";
				$onlineOrderItems .= '"paymentSubject":"SERVICE",';
				$onlineOrderItems .=
					'"name":"' . GetMessage("BIFIT_DELIVERY") . '",';
				$onlineOrderItems .= '"price":' . $order->delivery_cost . ",";
				$onlineOrderItems .=
					'"vatValue":' .
					($order->vat_mode == 1
						? $order->vat_value
						: $order->delivery_tax_value) .
					",";
				$onlineOrderItems .= '"quantity":1';
				$onlineOrderItems .= "}";
			}
		}

		// CREATE A USER IN THE CASH

		// address for POST request
		$url = "https://kassa.bifit.com/cashdesk-api/v1/protected/clients";

		// headers
		$headers = [
			"Content-Type: application/json",
			"Authorization: Bearer " . $token
		];

		// link data
		$data = "{";
		$data .= '"organizationId":"' . $order->org_id . '",';
		$data .= '"address":"' . $order->user_address . '",';
		$data .= '"phone":"' . $order->user_phone . '",';
		$data .= '"lastName":"' . $order->user_last_name . '",';
		$data .= '"firstName":"' . $order->user_name . '",';
		$data .= '"email":"' . $order->user_email . '"';
		$data .= "}";

		// log
		$log .= "## " . GetMessage("BIFIT_LOG_CHECK_CLIENT") . "\n";
		$log .= "## " . GetMessage("BIFIT_LOG_DATA_FOR_REQUEST") . ":" . "\n";
		$log .= $data . "\n";

		// trying to add a new client
		$client = json_decode(
			BifitKassaTools::__loadData($url, $headers, $data)
		);

		// such a client is already in the database
		if (is_object($client) && $client->type == "ERROR") {
			// address for POST request
			$url =
				"https://kassa.bifit.com/cashdesk-api/v1/protected/clients/list/read?organization_id=" .
				$order->org_id;

			// link data
			$data = false;

			// headers
			$headers = [
				"Content-Type: application/json",
				"Authorization: Bearer " . $token
			];

			// get all clients
			$clients = json_decode(
				BifitKassaTools::__loadData($url, $headers, $data)
			);

			foreach ($clients as $item) {
				// if some user data matches
				if ($item->phone == $order->user_phone) {
					$client = $item->id;
					$clientAddress = BifitKassaTools::__clearString(
						$item->address
					);
					$clientLastName = BifitKassaTools::__clearString(
						$item->lastName
					);
					$clientFirstName = BifitKassaTools::__clearString(
						$item->firstName
					);
					$clientPatronymic = $item->patronymic;
					$clientOrganizationInn = $item->organizationInn;
					$clientOrganizationName = $item->organizationName;
					$clientEmail = $item->email;
					$clientComment = BifitKassaTools::__clearString(
						$item->comment
					);
					$clientStatus = $item->status;
					$clientLoyalty = $item->loyalty;
					break;
				}
			}

			$cur =
				'{"address":"' .
				$clientAddress .
				'","lastName":"' .
				$clientLastName .
				'","firstName":"' .
				$clientFirstName .
				'","email":"' .
				$clientEmail .
				'"}';
			$ord =
				'{"address":"' .
				$order->user_address .
				'","lastName":"' .
				$order->user_last_name .
				'","firstName":"' .
				$order->user_name .
				'","email":"' .
				$order->user_email .
				'"}';

			// log
			$log .=
				"## " .
				GetMessage("BIFIT_LOG_CLIENT_ALREADY_EXIST") .
				$client .
				"\n";
			$log .=
				"## " .
				GetMessage("BIFIT_LOG_CLIENT_DATA_CHECK") .
				"\n" .
				"## " .
				GetMessage("BIFIT_LOG_CLIENT_DATA_ON_SERVER") .
				"\n" .
				$cur .
				"\n" .
				"## " .
				GetMessage("BIFIT_LOG_CLIENT_DATA_ON_ORDER") .
				"\n" .
				$ord .
				"\n";

			if (
				($order->user_email != "" &&
					$clientEmail != $order->user_email) ||
				($order->user_name != "" &&
					$clientFirstName != $order->user_name) ||
				($order->user_last_name != "" &&
					$clientLastName != $order->user_last_name) ||
				($order->user_address != "" &&
					$clientAddress != $order->user_address)
			) {
				$log .=
					"## " .
					GetMessage("BIFIT_LOG_CLIENT_DATA_BAD_START_UPDATE") .
					"\n";

				// UPDATE CLIENT DATA

				// address for PUT request
				$url =
					"https://kassa.bifit.com/cashdesk-api/v1/protected/clients/" .
					$client;

				// link data
				$data = "{";
				$data .= '"id":"' . $client . '",';
				$data .= '"organizationId":"' . $order->org_id . '",';
				$data .=
					'"address":"' .
					($order->user_address != "" &&
					$order->user_address != $clientAddress
						? $order->user_address
						: $clientAddress) .
					'",';
				$data .= '"phone":"' . $order->user_phone . '",';
				$data .=
					'"lastName":"' .
					($order->user_last_name != "" &&
					$order->user_last_name != $clientLastName
						? $order->user_last_name
						: $clientLastName) .
					'",';
				$data .=
					'"firstName":"' .
					($order->user_name != "" &&
					$order->user_name != $clientFirstName
						? $order->user_name
						: $clientFirstName) .
					'",';
				$data .= '"patronymic":"' . $clientPatronymic . '",';
				$data .= '"organizationInn":"' . $clientOrganizationInn . '",';
				$data .=
					'"organizationName":"' . $clientOrganizationName . '",';
				$data .=
					'"email":"' .
					($order->user_email != "" &&
					$order->user_email != $clientEmail
						? $order->user_email
						: $clientEmail) .
					'",';
				$data .=
					'"comment":"' .
					($order->user_comment != "" &&
					$order->user_comment != $clientComment
						? $order->user_comment
						: $clientComment) .
					'",';
				$data .= '"status":"' . $clientStatus . '"';
				//$data .= '"loyalty":"' . $clientLoyalty . '"';
				$data .= "}";

				$log .=
					"## " .
					GetMessage("BIFIT_LOG_CLIENT_DATA_UPDATE_DATA") .
					"\n" .
					$data .
					"\n";

				// headers
				$headers = [
					"Content-Type: application/json",
					"Authorization: Bearer " . $token
				];

				// put data
				$result = json_decode(
					BifitKassaTools::__loadData($url, $headers, $data, "PUT")
				);

				if (!is_object($result) && !is_array($result)) {
					$log .=
						"## " .
						GetMessage("BIFIT_LOG_CLIENT_DATA_UPDATE_GOOD") .
						"\n";
				} else {
					$log .=
						"## " .
						GetMessage("BIFIT_LOG_CLIENT_DATA_UPDATE_ERROR") .
						"\n" .
						BifitKassaTools::__checkResponce($result);
				}
			} else {
				$log .= "## " . GetMessage("BIFIT_LOG_CLIENT_DATA_OK") . "\n";
			}
		} else {
			// log
			$log .=
				"## " .
				GetMessage("BIFIT_LOG_ADDED_NEW_CLIENT_VS_ID") .
				$client .
				"\n";
		}

		// SENDING AN ORDER TO THE CASHIER

		// checking order availability
		// address for GET request
		$url =
			"https://kassa.bifit.com/cashdesk-api/v1/protected/online_orders/list/read/" .
			$client .
			"?organization_id=" .
			$order->org_id;

		// link data
		$data = false;

		// headers
		$headers = [
			"Content-Type: application/x-www-form-urlencoded",
			"Authorization: Bearer " . $token
		];

		// get orders
		$orders = json_decode(
			BifitKassaTools::__loadData($url, $headers, $data, "GET")
		);

		if (is_object($orders) && $orders->status == 500) {
			$log .=
				"## " .
				GetMessage("BIFIT_LOG_GET_ORDER_DATA_ERROR") .
				"\n" .
				BifitKassaTools::__checkResponce($orders);
			BifitKassaTools::__saveLogfile($log, $order->number);

			return false;
		}

		// params
		$order_id = null;
		$delivery_type = "";
		$delivery_time = 0;
		$currentStatusType = "";
		$order_deleted = null;

		// log
		$log .= "## " . GetMessage("BIFIT_LOG_CHECK_CURR_ORDER") . "\n";

		// trans
		$l = [
			"COURIER" => GetMessage("BIFIT_LOG_TYPE_COURIER"),
			"SELF" => GetMessage("BIFIT_LOG_TYPE_SELF")
		];

		// search order by id
		foreach ($orders as $key => $item) {
			if ($item->externalId == $order->number) {
				$order_id = $item->id;
				$delivery_type = strtoupper($item->deliveryType);
				$delivery_time = isset($item->deliveryTime)
					? $item->deliveryTime
					: 0;
				$currentStatusType = strtoupper($item->currentStatusType);

				// log
				$log .=
					"## " .
					GetMessage("BIFIT_LOG_THIS_ORDER_ALREADY_EXIST") .
					$order_id .
					". " .
					GetMessage("BIFIT_LOG_DELIVERY_TYPE") .
					$l[$delivery_type] .
					"." .
					"\n";

				break;
			}
		}

		// log
		if (!$order_id) {
			$log .= "## " . GetMessage("BIFIT_LOG_CURR_ORDER_NOT_FOUND") . "\n";
		}

		// type of delivery
		$deliveryType = $delivery_type != "" ? $delivery_type : "SERVICE";

		if (
			isset($order->courier) &&
			in_array($order->delivery_id, $order->courier)
		) {
			$deliveryType = "COURIER";
		} elseif (
			isset($order->self) &&
			in_array($order->delivery_id, $order->self)
		) {
			$deliveryType = "SELF";
		}

		// assign "is_refused"
		$is_refused =
			isset($order_id) &&
			isset($order->refused_statuses) &&
			in_array($order->status_id, $order->refused_statuses) &&
			$order->use_refused
				? 1
				: null;

		if ($deliveryType == "SERVICE") {
			// log
			$txt = !$order_id
				? GetMessage("BIFIT_LOG_FAILED_TO_SAVE_CURR_ORDER")
				: GetMessage("BIFIT_LOG_FAILED_TO_UPDATE_CURR_ORDER");
			$txt .= " " . GetMessage("BIFIT_LOG_FAILED_TO_SERVICE_TEXT");
			$log .= $txt != "" ? "## " . $txt . "\n" : "";
		} elseif (!$order_id || $is_refused) {
			// SAVE || REFUSE

			// address for request
			$url =
				"https://kassa.bifit.com/cashdesk-api/v1/protected/online_orders";

			// headers
			$headers = [
				"Content-Type: application/json",
				"Authorization: Bearer " . $token
			];

			// data
			if ($is_refused) {
				// log
				$log .=
					"## " . GetMessage("BIFIT_LOG_START_ORDER_UPDATE") . "\n";

				$url .= "/" . $order_id . "/status";
				$type = "PUT";
				$statusType = $delivery_type == "SELF" ? "REFUSED" : "DELETED";

				// Basic data
				$data = "{";
				$data .= '"onlineOrderId":"' . $order_id . '",';
				$data .= '"statusType":"' . $statusType . '",';
				$data .= '"statusDate":0,';
				$data .= '"statusInfo":""';
				$data .= "}";
			} elseif (!$order_id) {
				// log
				$log .= "## " . GetMessage("BIFIT_LOG_START_NEW_ORDER") . "\n";

				// data
				$type = "POST";
				$statusType =
					$deliveryType == "SELF"
						? "ACCEPTED"
						: strtoupper($order->new_status);

				// timestamp
				$timestamp = strtotime($order->created);
				$orderTime =
					strlen($timestamp) == 10 ? $timestamp * 1000 : $timestamp;

				// Basic data
				$onlineOrder = "{";
				$onlineOrder .= '"organizationId":"' . $order->org_id . '",';
				$onlineOrder .= '"tradeObjectId":"' . $order->trade_id . '",';
				$onlineOrder .=
					'"taxSystem":"' . strtoupper($order->tax_system) . '",';
				$onlineOrder .= '"externalId":"' . $order->number . '",';
				$onlineOrder .= '"deliveryType":"' . $deliveryType . '",';
				$onlineOrder .=
					'"deliveryAmount":"' . $order->delivery_cost . '",';
				$onlineOrder .= '"paid":false,';
				$onlineOrder .= '"orderTime":' . $orderTime . ",";
				$onlineOrder .= '"clientId":' . $client . ",";
				$onlineOrder .=
					'"address":"' . addslashes($order->user_address) . '",';
				$onlineOrder .=
					'"comment":"' .
					htmlspecialchars(
						trim($order->user_comment . " " . $order->comment)
					) .
					'",';
				$onlineOrder .=
					'"responsiblePersonLogin":"' . $order->login . '",';
				$onlineOrder .= '"totalAmount":' . (float) $order->total . ",";
				$onlineOrder .= '"currentStatusType":"' . $statusType . '",';
				$onlineOrder .= '"currentStatusTime":0';
				$onlineOrder .= "}";

				$data = "{";
				$data .= '"onlineOrder":' . $onlineOrder . ",";
				$data .= '"onlineOrderItems":[' . $onlineOrderItems . "]";
				//$data .= ',"onlineOrderPayments":[]';
				$data .= "}";
			}

			// log
			$log .=
				"## " .
				GetMessage("BIFIT_LOG_ORDER_DATA_FOR_REQUEST") .
				":" .
				"\n";
			$log .= $data . "\n";

			// get the ID of the document being processed (id)
			$order_id = json_decode(
				BifitKassaTools::__loadData($url, $headers, $data, $type)
			);

			// log
			if (!is_object($order_id) && !is_array($order_id)) {
				if ($is_refused && $statusType == "DELETED") {
					$order_deleted = true;
					$log .=
						"## " .
						GetMessage("BIFIT_LOG_SUCC_ORDER_DEL") .
						$order_id .
						"\n";
				} elseif ($is_refused && $statusType == "REFUSED") {
					$log .=
						"## " .
						GetMessage("BIFIT_LOG_SUCC_ORDER_REFUSED") .
						$order_id .
						"\n";
				} else {
					$log .=
						"## " .
						GetMessage("BIFIT_LOG_SUCC_ORDER_SAVE") .
						$order_id .
						"\n";
				}
			} else {
				$log .=
					"## " .
					GetMessage("BIFIT_LOG_ERR_IN_SAVE_ORDER_PROC") .
					":" .
					"\n";
				$log .= BifitKassaTools::__checkResponce($order_id) . "\n";
			}

			// SEND PAYMENT INFORMATION

			if (
				is_numeric($order_id) &&
				!$order_deleted &&
				in_array(strtoupper($order->calc_method), [
					"REPAID_EXPENSE",
					"PREPAY_FULL",
					"PREPAY_PARTIAL",
					"FULL_PAY"
				]) &&
				($order->paid == "Y" || $order->sum_paid > 0)
			) {
				// get order payments
				$orderTransIds = BifitKassaTools::__getOrderTransIds(
					$order_id,
					$order->org_id,
					$token
				);

				// params
				$calc_method = strtoupper($order->calc_method);
				$total =
					$order->sum_paid &&
					$order->sum_paid > 0 &&
					$order->total > 0
						? $order->sum_paid
						: $order->total;
				$ok_payments = $order->ok_payments;
				$k = (int) (count($ok_payments) - 1);
				$last_ok_pm = $ok_payments[$k];

				if (count($order->payments)) {
					if (count($order->payments) > 1) {
						$calc_method = "PREPAY_PARTIAL";
					} elseif (count($order->payments) == 1) {
						$calc_method = $order->avans_calc_method;
					}

					foreach ($order->payments as $item) {
						if ($item["PAID"] != "Y") {
							continue;
						}

						if (
							$order->bonus &&
							in_array(
								$item["PAY_SYSTEM_ID"],
								$order->pm_bonus_ids
							) &&
							$order->total > 0
						) {
							continue;
						}

						// check item price
						$item_total =
							$order->total == 0 ? $order->total : $item["SUM"];

						// correct item summ
						if (
							$order->paid == "Y" &&
							$order->total > 0 &&
							$order->total != $order->sum_paid &&
							$last_ok_pm["XML_ID"] == $item["XML_ID"]
						) {
							$item_total =
								$item["SUM"] +
								(float) ($order->total - $order->sum_paid);
						}

						// log
						$log .=
							"## " .
							GetMessage("BIFIT_LOG_START_PAY_DATA_PROC") .
							"\n";

						// address for request
						$url =
							"https://kassa.bifit.com/cashdesk-api/v1/protected/online_orders/payments";

						// headers
						$headers = [
							"Content-Type: application/json",
							"Authorization: Bearer " . $token
						];

						// data
						$type = "POST";
						$data = "{";
						$data .=
							'"onlineOrderExternalId":"' . $order->number . '",';

						if (
							!is_object($order_id) &&
							!is_array($order_id) &&
							is_numeric($order_id)
						) {
							$data .= '"onlineOrderId":' . (int) $order_id . ",";
						}

						$data .= '"transactionId":"' . $item["XML_ID"] . '",';
						$data .= '"organizationId":"' . $order->org_id . '",';
						$data .= '"calculationMethod":"' . $calc_method . '",';
						$data .= '"total":' . $item_total;
						$data .= "}";

						// log
						$log .=
							"## " .
							GetMessage("BIFIT_LOG_PAY_DATA_FOR_REQUEST") .
							":" .
							"\n";
						$log .= $data . "\n";
						$log .=
							"## " .
							GetMessage("BIFIT_LOG_PAY_DATA_CHECK") .
							"\n";

						// check exist
						if (
							count($orderTransIds) &&
							in_array($item["XML_ID"], $orderTransIds)
						) {
							// log
							$log .=
								"## " .
								GetMessage("BIFIT_LOG_PAY_DATA_CHECK_EXIST") .
								"\n";
							continue;
						}

						// log
						$log .=
							"## " .
							GetMessage("BIFIT_LOG_PAY_DATA_CHECK_NO_EXIST") .
							"\n";

						// get the ID of the document being processed (id)
						$response = json_decode(
							BifitKassaTools::__loadData(
								$url,
								$headers,
								$data,
								$type
							)
						);

						// log
						if (!is_object($response) && !is_array($response)) {
							$log .=
								"## " .
								GetMessage("BIFIT_LOG_SUCC_SEND_PAY_DATA") .
								$response .
								"\n";
						} else {
							$log .=
								"## " .
								GetMessage("BIFIT_LOG_ERR_WHEN_SEND_PAY_DATA") .
								":" .
								"\n";
							$log .=
								BifitKassaTools::__checkResponce($response) .
								"\n";
						}

						if ($order->total == 0) {
							break;
						}
					}
				}
			}
		} elseif (
			isset($order_id) &&
			!is_object($order_id) &&
			!is_array($order_id) &&
			is_numeric($order_id)
		) {
			// log
			$log .=
				"## " .
				GetMessage("BIFIT_LOG_START_GLOBAL_ORDER_UPDATE") .
				"\n";

			// address for request
			$url =
				"https://kassa.bifit.com/cashdesk-api/v1/protected/online_orders/" .
				$order_id .
				"/full";

			// headers
			$headers = [
				"Content-Type: application/json",
				"Authorization: Bearer " . $token
			];

			// onlineOrder
			$timestamp = strtotime($order->created);
			$orderTime =
				strlen($timestamp) == 10 ? $timestamp * 1000 : $timestamp;

			$onlineOrder = '"organizationId":"' . $order->org_id . '",';
			$onlineOrder .= '"tradeObjectId":"' . $order->trade_id . '",';
			$onlineOrder .=
				'"taxSystem":"' . strtoupper($order->tax_system) . '",';
			$onlineOrder .= '"externalId":"' . $order->number . '",';
			$onlineOrder .= '"deliveryType":"' . $delivery_type . '",';
			$onlineOrder .=
				'"deliveryAmount":' . (float) $order->delivery_cost . ",";

			if ((int) $delivery_time > 0) {
				$onlineOrder .= '"deliveryTime": ' . $delivery_time . ",";
			}

			$onlineOrder .=
				'"paid":' . ($order->paid == "Y" ? "true" : "false") . ",";
			$onlineOrder .= '"orderTime":' . $orderTime . ",";
			$onlineOrder .= '"clientId":' . $client . ",";
			$onlineOrder .=
				'"address":"' . addslashes($order->user_address) . '",';
			$onlineOrder .= '"responsiblePersonLogin":"' . $order->login . '",';
			$onlineOrder .= '"totalAmount":' . (float) $order->total . ",";
			$onlineOrder .=
				'"comment":"' .
				htmlspecialchars(
					trim($order->user_comment . " " . $order->comment)
				) .
				'",';
			$onlineOrder .= '"currentStatusType":"' . $currentStatusType . '",';
			$onlineOrder .= '"currentStatusTime":0,';
			$onlineOrder .= '"cashierName":"' . $order->cashier_name . '",';
			$onlineOrder .= '"clientName":"' . $order->user_name . '"';

			// onlineOrderItems
			$onlineOrderItems = "";

			// products
			foreach ($order->products as $key_id => $product) {
				$onlineOrderItems .= $onlineOrderItems != "" ? ", " : "";
				$onlineOrderItems .= "{";
				$onlineOrderItems .=
					'"nomenclatureId":' . $product->nomenclature_id . ",";
				$onlineOrderItems .= '"paymentSubject":"PRODUCT",';
				$onlineOrderItems .= '"name":"' . $product->name . '",';
				$onlineOrderItems .= '"price":' . (float) $product->price . ",";
				$onlineOrderItems .=
					'"vatValue":' .
					($order->vat_mode == 1
						? $order->vat_value
						: $product->tax_value) .
					",";
				$onlineOrderItems .= '"quantity":' . $product->count . ",";
				$onlineOrderItems .= '"vendorCode":"' . $product->code . '"';
				$onlineOrderItems .= "}";
			}

			// delivery
			if ($order->delivery_cost > 0) {
				$onlineOrderItems .= $onlineOrderItems != "" ? "," : "";
				$onlineOrderItems .= "{";
				$onlineOrderItems .=
					'"nomenclatureId":' .
					$order->delivery_nomenclature_id .
					",";
				$onlineOrderItems .= '"paymentSubject":"SERVICE",';
				$onlineOrderItems .=
					'"name":"' . GetMessage("BIFIT_DELIVERY") . '",';
				$onlineOrderItems .=
					'"price":' . (float) $order->delivery_cost . ",";
				$onlineOrderItems .=
					'"vatValue":' .
					($order->vat_mode == 1
						? $order->vat_value
						: $order->delivery_tax_value) .
					",";
				$onlineOrderItems .= '"quantity":1';
				$onlineOrderItems .= "}";
			}

			// onlineOrderPayments
			$onlineOrderPayments = "";

			if (
				in_array(strtoupper($order->calc_method), [
					"REPAID_EXPENSE",
					"PREPAY_FULL",
					"PREPAY_PARTIAL",
					"FULL_PAY"
				]) &&
				($order->paid == "Y" || $order->sum_paid > 0)
			) {
				// get order payments
				$orderTransIds = BifitKassaTools::__getOrderTransIds(
					$order_id,
					$order->org_id,
					$token
				);

				// params
				$calc_method = strtoupper($order->calc_method);
				$total =
					$order->sum_paid &&
					$order->sum_paid > 0 &&
					$order->total > 0
						? $order->sum_paid
						: $order->total;
				$ok_payments = $order->ok_payments;
				$k = (int) (count($ok_payments) - 1);
				$last_ok_pm = $ok_payments[$k];

				if (count($order->payments)) {
					if (count($order->payments) > 1) {
						$calc_method = "PREPAY_PARTIAL";
					} elseif (
						count($order->payments) == 1 &&
						$order->sum_paid > 0 &&
						$order->sum_paid < $order->total
					) {
						$calc_method = $order->avans_calc_method;
					}

					foreach ($order->payments as $item) {
						if ($item["PAID"] != "Y") {
							continue;
						}

						if (
							$order->bonus &&
							in_array(
								$item["PAY_SYSTEM_ID"],
								$order->pm_bonus_ids
							) &&
							$order->total > 0
						) {
							continue;
						}

						// check item price
						$item_total =
							$order->total == 0 ? $order->total : $item["SUM"];

						// correct item summ
						if (
							$order->paid == "Y" &&
							$order->total > 0 &&
							$order->total != $order->sum_paid &&
							$last_ok_pm["XML_ID"] == $item["XML_ID"]
						) {
							$item_total =
								$item["SUM"] +
								(float) ($order->total - $order->sum_paid);
						}

						$payments_kassa_id = null;

						if (
							count($orderTransIds) &&
							in_array($item["XML_ID"], $orderTransIds)
						) {
							$payments_kassa_id = array_search(
								$item["XML_ID"],
								$orderTransIds
							);
						}

						$onlineOrderPayments .=
							$onlineOrderPayments != "" ? ", " : "";
						$onlineOrderPayments .= "{";
						$onlineOrderPayments .=
							'"calculationMethod":"' .
							strtoupper($calc_method) .
							'",';

						if (
							isset($payments_kassa_id) &&
							!is_object($payments_kassa_id) &&
							!is_array($payments_kassa_id) &&
							is_numeric($payments_kassa_id)
						) {
							$onlineOrderPayments .=
								'"id":' . $payments_kassa_id . ",";
						}

						$onlineOrderPayments .=
							'"onlineOrderExternalId":"' . $order->number . '",';
						$onlineOrderPayments .=
							'"onlineOrderId":' . (int) $order_id . ",";
						$onlineOrderPayments .=
							'"organizationId":"' . $order->org_id . '",';
						$onlineOrderPayments .=
							'"total":' . (float) $item_total . ",";
						$onlineOrderPayments .=
							'"transactionId":"' . $item["XML_ID"] . '"';
						$onlineOrderPayments .= "}";
					}
				}
			}

			// client
			$client = '"organizationId":"' . $order->org_id . '",';
			$client .=
				'"address":"' .
				($order->user_address != "" &&
				$order->user_address != $clientAddress
					? $order->user_address
					: $clientAddress) .
				'",';
			$client .= '"phone":"' . $order->user_phone . '",';
			$client .=
				'"lastName":"' .
				($order->user_last_name != "" &&
				$order->user_last_name != $clientLastName
					? $order->user_last_name
					: $clientLastName) .
				'",';
			$client .=
				'"firstName":"' .
				($order->user_name != "" &&
				$order->user_name != $clientFirstName
					? $order->user_name
					: $clientFirstName) .
				'",';
			$client .=
				'"email":"' .
				($order->user_email != "" && $order->user_email != $clientEmail
					? $order->user_email
					: $clientEmail) .
				'"';

			// cashierInfos
			$cashierInfos = '"firstName":"' . $order->cashier_name . '",';
			$cashierInfos .=
				'"inn":"' .
				BifitKassaTools::__checkInn($order->cashier_inn, 1) .
				'"';

			// data
			$data = "{";
			$data .= '"onlineOrder": {' . $onlineOrder . "},";
			$data .= '"onlineOrderItems": [' . $onlineOrderItems . "],";
			$data .= '"onlineOrderPayments": [' . $onlineOrderPayments . "],";
			$data .= '"responsiblePerson": {},';
			$data .= '"client": {' . $client . "},";
			$data .= '"organizationId": "' . $order->org_id . '",';
			$data .= '"cashierInfos": [{' . $cashierInfos . "}]";
			$data .= "}";

			// log
			$log .=
				"## " .
				GetMessage("BIFIT_LOG_GLOBAL_ORDER_DATA_FOR_REQUEST") .
				":" .
				"\n";
			$log .= $data . "\n";

			// update order by order_id
			$result = json_decode(
				BifitKassaTools::__loadData($url, $headers, $data, "PUT")
			);

			// log
			if (!is_object($result) && !is_array($result)) {
				$log .=
					"## " .
					GetMessage("BIFIT_LOG_GLOBAL_ORDER_DATA_UPDATE_RESULT_OK") .
					"!" .
					"\n";
			} else {
				$log .=
					"## " .
					GetMessage(
						"BIFIT_LOG_GLOBAL_ORDER_DATA_UPDATE_RESULT_BAD"
					) .
					" " .
					BifitKassaTools::__checkResponce($result) .
					"\n";
			}
		}

		// save log
		BifitKassaTools::__saveLogfile($log, $order->number);
	}

	static function __doFiscalize($order)
	{
		Bitrix\Main\Localization\Loc::loadMessages(__FILE__);

		$mid = self::MODULE_ID;

		// 1. AUTHORIZATION
		//    process necessary to connect a client to fiscal processing

		// address for POST request
		$url =
			"https://fp" .
			$order->test .
			".bifit.com/processing-api/oauth/token";

		// link data
		$data = "token=" . $order->token;
		$data .= "&client_id=processing-connector-token";
		$data .= "&client_secret=processing-connector-token";
		$data .= "&grant_type=token";

		// headers
		$headers = ["Content-Type: application/x-www-form-urlencoded"];

		// decode the received json response
		$bifit = json_decode(
			BifitKassaTools::__loadData($url, $headers, $data)
		);

		// 2. FORMATION OF THE CONTENT OF A CHECK
		//    formation of information for further sending for fiscalization

		// params
		$order_total =
			$order->is_avans && $order->sum_paid > 0
				? $order->sum_paid
				: $order->total;
		$calc_method = $order->is_avans
			? strtoupper($order->avans_calc_method)
			: strtoupper($order->calc_method);
		$payment_type = $order->is_avans
			? strtoupper($order->avans_payment_type)
			: strtoupper($order->payment_type);
		$payments = '"' . $payment_type . '":' . $order_total;

		// avans
		if (
			!$order->is_avans &&
			count($order->payments) > 1 &&
			$order_total > 0
			// && $order_total <= $order->sum_paid
		) {
			$i = 0;
			$payments = "";

			foreach ($order->payments as $item) {
				if ($item["PAID"] != "Y") {
					continue;
				}

				if (
					$order->bonus &&
					in_array($item["PAY_SYSTEM_ID"], $order->pm_bonus_ids) &&
					$order->total > $order->bonus_summ
				) {
					continue;
				}

				$payment_type =
					$i == 0 ? "PREPAY" : strtoupper($order->payment_type);
				$payments .= $payments != "" ? "," : "";
				$payments .= '"' . $payment_type . '":' . $item["SUM"];
				$i++;
			}
		}

		// check parameters
		$order_products = $order->is_avans
			? $order->avans_products
			: $order->products;
		$items = "";

		// generate check data
		foreach ($order_products as $key_id => $product) {
			// display the product data in the check
			$items .= $items != "" ? "," : "";
			$items .= "{";
			$items .= '"calculationMethod":"' . strtoupper($calc_method) . '",';
			$items .= '"paymentSubject":"' . $product->paymentsubject . '",';
			$items .= '"name":"' . $product->name . '",';
			$items .= '"price":' . $product->price . ",";
			$items .= '"quantity":' . $product->count . ",";
			$items .=
				$order->vat_mode == 1
					? '"vat":"' . strtoupper($order->vat) . '",'
					: '"vat":"' . strtoupper($product->tax) . '",';
			$items .=
				'"total":' .
				BifitKassaTools::__formatprc($product->price * $product->count);
			$items .= "}";
		}

		// delivery
		if ($order->delivery_cost > 0 && !$order->is_avans) {
			// display the data of the delivery method in the check
			$items .= $items != "" ? "," : "";
			$items .= "{";
			$items .= '"calculationMethod":"' . strtoupper($calc_method) . '",';
			$items .= '"paymentSubject":"SERVICE",';
			$items .= '"name":"' . GetMessage("BIFIT_DELIVERY") . '",';
			$items .= '"price":' . $order->delivery_cost . ",";
			$items .= '"quantity":1,';
			$items .=
				$order->vat_mode == 1
					? '"vat":"' . strtoupper($order->vat) . '",'
					: '"vat":"' . strtoupper($order->delivery_tax) . '",';
			$items .= '"total":' . $order->delivery_cost;
			$items .= "}";
		}

		// form a check
		$receipt = "{";
		$receipt .=
			'"type":"' .
			strtoupper($order->is_avans ? 'SALE' : $order->receipt_type) .
			'",';
		$receipt .= '"taxSystem":"' . strtoupper($order->tax_system) . '",';
		$receipt .= '"cashier":{';
		$receipt .= '"name":"' . $order->cashier_name . '",';
		$receipt .=
			'"inn":"' .
			BifitKassaTools::__checkInn($order->cashier_inn, 1) .
			'"';
		$receipt .= "},";
		$receipt .= '"client":{';
		$receipt .= $order->fio
			? '"name":"' .
				trim($order->user_name . " " . $order->user_last_name) .
				'",'
			: "";
		$receipt .= '"address":"' . $order->address . '"';
		$receipt .= "},";
		$receipt .= '"paymentAddress":"' . $order->payment_address . '",';
		$receipt .= '"items":[' . $items . "],";
		$receipt .= '"total":' . $order_total . ",";
		$receipt .= '"payments":{' . $payments . "}";
		$receipt .= "}";

		// 3. SENDING A CHECK TO FISCAL PROCESSING
		//    transfer of the content of the check to fiscal processing for further fiscalization

		// address for POST request
		$url =
			"https://fp" .
			$order->test .
			".bifit.com/processing-api/protected/documents/registration/receipts";

		// check data
		$data = $receipt;

		// Idempotency-Key
		$o = [];
		$o["number"] = $order->number;
		$o["summ"] = $order_total;
		$o["cms"] = "Bitrix";
		$idemp_key = BifitKassaTools::__createGUID($o);

		// headers
		$headers = [
			"Content-Type: application/json",
			"Authorization: Bearer " . $bifit->access_token,
			"Idempotency-Key: " . $idemp_key
		];

		// get the ID of the document being processed (id)
		$receipt_id = json_decode(
			BifitKassaTools::__loadData($url, $headers, $data)
		);

		// 4. RECEIVING DOCUMENT BY ID
		//    obtaining a document with fiscal characteristics by its number

		// options
		$doc = "";

		if (!is_object($receipt_id)) {
			// link for GET request
			$url =
				"https://fp" .
				$order->test .
				".bifit.com/processing-api/protected/documents/" .
				$receipt_id;

			// headers
			$headers = ["Authorization: Bearer " . $bifit->access_token];

			// we get the document
			$doc = json_decode(
				BifitKassaTools::__loadData($url, $headers, false, "GET")
			);
		}

		// LINK TO THE CHECK

		// we form a link to the check
		$receipt_link = !is_object($receipt_id)
			? "https://fp" .
				$order->test .
				".bifit.com/processing-api/receipts/" .
				$idemp_key
			: "XXX";

		// OPERATION HISTORY

		// update data
		$order->idemp_key = $idemp_key;
		$order->receipt = $receipt;
		$order->receipt_link = $receipt_link;

		// save the operation to history
		BifitKassaTools::__saveReport($order, $receipt_id, $doc, $bifit);
	}

	public static function __preActivation($id, $arFields)
	{
		if (gettype($id) == "object" || !CModule::IncludeModule("sale")) {
			return;
		}

		$mid = self::MODULE_ID;
		$data = BifitKassaTools::__getOrderData($arFields);
		$data->test = COption::GetOptionString($mid, "test") ? "-test" : "";
		$data->token =
			$data->test != ""
				? "P5cKbUUD9uSSrSlGdzspLblvBnD0GzTAE0cLmAPSEMxJ79DtLE"
				: COption::GetOptionString($mid, "token");
		$status_id = $arFields["STATUS_ID"];
		$payment_id = $data->payment_id;
		$paid = $data->paid;

		// UNIQUE - check unique transactions param
		$uniq = COption::GetOptionString($mid, "uniq", 1);

		// UNIQUE - param enable
		if ($uniq) {
			// UNIQUE - get types for uniq
			$types = BifitKassaTools::__getOrderTypes($id);

			// UNIQUE - set types array
			$types_arr = [
				"0" => "AVANS",
				"1" => "SALE",
				"2" => "SALE_RETURN",
				"3" => "PURCHASE",
				"4" => "PURCHASE_RETURN"
			];
		}

		// no action duplicates
		$no_duplicates = COption::GetOptionString($mid, "no_duplicates", 1);

		if ($no_duplicates) {
			$session = ("\Bitrix\Main\Application")
				::getInstance()
				->getSession();
		}

		// iterating over the document type tabs
		for ($i = 0; $i <= 5; $i++) {
			// will find out whether it is necessary to use this type of document
			$use_type = COption::GetOptionString($mid, "use_type_" . $i)
				? 1
				: null;
			// if need to use
			if ($use_type) {
				// get parameters
				/// order paid (yes / no / disregard) (2/1/0)
				$must_paid = COption::GetOptionString($mid, "order_paid_" . $i);
				//// if ZERO param paid = paritial paid
				if ($i == 0 && $must_paid == "2" && $data->sum_paid > 0) {
					$must_paid = "0";
				}
				/// If Sale and order have one payments < total
				if (
					$i == 1 &&
					count($data->payments) &&
					$data->sum_paid > 0 &&
					$data->sum_paid < $data->total
				) {
					continue;
				}
				/// selected order status (s)
				$pm_statuses = json_decode(
					COption::GetOptionString($mid, "pm_statuses_" . $i)
				);
				/// selected payment method (s)
				$pm_ids = json_decode(
					COption::GetOptionString($mid, "pm_ids_" . $i)
				);
				/// use refused
				$use_refused = COption::GetOptionString($mid, "lk_use_refused");
				/// connection tab
				if ($i == 5 && $use_refused) {
					//// selected refused status (s)
					$lk_refused_statuses = json_decode(
						COption::GetOptionString($mid, "lk_refused_statuses")
					);
					/// if in array merge status arrays
					if (in_array($status_id, $lk_refused_statuses)) {
						$pm_statuses = array_merge(
							$pm_statuses,
							$lk_refused_statuses
						);
					}
				}

				// If the payment method and order status match
				if (
					isset($pm_ids) &&
					isset($pm_statuses) &&
					in_array($payment_id, $pm_ids) &&
					in_array($status_id, $pm_statuses) &&
					$data->token != ""
				) {
					// if "order paid" is selected,
					// and it is NOT paid, then we check the following tab
					if ($must_paid == "2" && $paid != "Y") {
						continue;
					}

					// if "order not paid" is selected,
					// and it is paid, then we check the next tab
					if ($must_paid == "1" && $paid != "N") {
						continue;
					}

					// check for duplicates
					if ($no_duplicates) {
						$act_name = "bifit_act_" . $i . "_" . $id;

						if ($session->has($act_name)) {
							if (
								$session->get($act_name) >
								strtotime(date("Y-m-d H:i:s"))
							) {
								continue;
							}
						}
					}

					// update data
					$arr_rec_type = [
						"0" => "AVANS",
						"1" => "SALE",
						"2" => "SALE_RETURN",
						"3" => "PURCHASE",
						"4" => "PURCHASE_RETURN"
					];
					$data->receipt_type = $arr_rec_type[$i];
					$data->tax_system = COption::GetOptionString(
						$mid,
						"tax_system_" . $i
					);

					// UNIQUE
					// if uniqueness is selected and
					// this is not a connection with the account and
					// the reports contain the same (type, number) successfully completed order,
					// then the next tab
					if (
						$uniq &&
						$i != 5 &&
						count($types) &&
						in_array($types_arr[$i], $types)
					) {
						continue;
					}

					// main processes
					if ($i == 5) {
						// update data
						$data->login = BifitKassaTools::__checkPhone(
							COption::GetOptionString($mid, "login")
						);
						$data->password = COption::GetOptionString(
							$mid,
							"password"
						);
						$data->org_id = COption::GetOptionString(
							$mid,
							"organization"
						);
						$data->trade_id = COption::GetOptionString(
							$mid,
							"trade_object"
						);
						$data->courier = json_decode(
							COption::GetOptionString(
								$mid,
								"lk_courier_delivery"
							)
						);
						$data->self = json_decode(
							COption::GetOptionString($mid, "lk_self_delivery")
						);
						$data->new_status = COption::GetOptionString(
							$mid,
							"lk_new_courier_order_status"
						);
						$data->use_refused = COption::GetOptionString(
							$mid,
							"lk_use_refused"
						);
						$data->refused_statuses = json_decode(
							COption::GetOptionString(
								$mid,
								"lk_refused_statuses"
							)
						);
						// correcting Avans value
						$data->avans_calc_method =
							$data->avans_calc_method == 'AVANS'
								? 'REPAID_EXPENSE'
								: $data->avans_calc_method;
						// do
						self::__saveOrder($data);
					} else {
						// is Avans ?
						$data->is_avans = $i == 0 && $data->avans ? 1 : null;
						// do
						self::__doFiscalize($data);
					}

					// save to session and check old values
					if ($no_duplicates) {
						if (!$session->has("bifit_acts")) {
							$session["bifit_acts"] = [];
						} else {
							$arr = $session->get("bifit_acts");

							if (is_array($arr) && count($arr)) {
								foreach ($arr as $k => $v) {
									$a_n = "bifit_act_" . $v;

									if ($session->has($a_n)) {
										if (
											strtotime(date("Y-m-d H:i:s")) >
											$session->get($a_n)
										) {
											$session->remove($a_n);
											unset($session["bifit_acts"][$k]);
										}
									}
								}
							}
						}

						$session["bifit_acts"][] = $i . "_" . $id;
						$session->set(
							$act_name,
							strtotime(date("Y-m-d H:i:s") . "+5 seconds")
						);
					}
				}
			}
		}
	}

	public static function handleOrderAdd($id, $arFields)
	{
		self::__preActivation($id, $arFields);
	}

	public static function handleOrderUpdate($id, $arFields)
	{
		self::__preActivation($id, $arFields);
	}
}
